# -*- coding: utf-8 -*-

'''
High-level editor interface that communicates with underlying editor (like
Espresso, Coda, etc.) or browser.
Basically, you should call <code>set_context(obj)</code> method to
set up undelying editor context before using any other method.

This interface is used by <i>zen_actions.py</i> for performing different
actions like <b>Expand abbreviation</b>

@example
from zencoding.interface.editor import ZenEditor
editor = ZenEditor(context)
//now you are ready to use editor object
editor.get_selection_range();

@author Sergey Chikuyonok (serge.che@gmail.com)
@link http://chikuyonok.ru

# CotEditor 専用に変更
# 要 Appscript
# 修正:Tachi.K
# 修正日:2011/03/17

'''
import appscript
import commands
import os.path
import re
import sys
import zencoding.utils
import zencoding.zen_settings


class ZenEditor():
	def __init__(self, context=None):
		print "__init__"
		self._content                  = u''
		self._doc                      = None
		self._original_selection_range = None
		self._forcedProfile            = ''
		
		# ドキュメントオブジェクトを得る
		self._doc = appscript.app('CotEditor').documents[1]
		self._original_selection_range = self._doc.selection.range.get()
		
		# タブ設定を取得する
		try:
			cotEditorAutoExpandTab = commands.getoutput("""defaults read com.aynimac.CotEditor autoExpandTab""")
		except:
			cotEditorAutoExpandTab = "0"
		
		# タブを空白に変換する場合、空白何個分かを取得する
		if (cotEditorAutoExpandTab == '1'):
			try:
				cotEditorTabWidth = commands.getoutput("""defaults read com.aynimac.CotEditor tabWidth""")
			except:
				cotEditorTabWidth = "4"
			indentation = " " * int(cotEditorTabWidth)
		
			# タブ設定を zen_settings に反映する
			zencoding.utils.set_variable('indentation', indentation)
		else:
			zencoding.utils.set_variable('indentation', '\t')
		
	def set_context(self, context=None):
		"""
		Setup underlying editor context. You should call this method
		<code>before</code> using any Zen Coding action.
		@param context: context object
		"""
		print "set_context"
		self._content = context

	def get_selection_range(self):
		"""
		Returns character indexes of selected text
		@return: list of start and end indexes
		@example
		start, end = zen_editor.get_selection_range();
		print('%s, %s' % (start, end))
		"""
		print "get_selection_range"
		# 選択範囲を得る
		return self._original_selection_range[0], self._original_selection_range[0] + self._original_selection_range[1]

	def create_selection(self, start, end=None):
		"""
		Creates selection from <code>start</code> to <code>end</code> character
		indexes. If <code>end</code> is ommited, this method should place caret
		and <code>start</code> index
		@type start: int
		@type end: int
		@example
		zen_editor.create_selection(10, 40)
		# move caret to 15th character
		zen_editor.create_selection(15)
		"""
		print("create_selection[%d, %d]" % (start, end))

		# 選択範囲を設定する
		if end == None :
			length = 0
		else :
			length = end-start
		
		self._doc.selection.range.set(to=[start, length])

	def get_current_line_range(self):
		"""
		Returns current line's start and end indexes
		@return: list of start and end indexes
		@example
		start, end = zen_editor.get_current_line_range();
		print('%s, %s' % (start, end))
		"""
		print "get_current_line_range"
		
		# 現在の行を選択する。
		self._doc.selection.line_range.set(to=self._doc.selection.line_range.get())
		
		# 現在行の選択範囲を得る
		pos = self._doc.selection.range.get()
		start = int(pos[0])
		end   = start + int(pos[1])
		
		# 選択範囲を戻す
		self._doc.selection.range.set(to=self._original_selection_range)
		
		return start, end

	def get_caret_pos(self):
		""" Returns current caret position """
		print "get_caret_pos"
		# 現在のキャレット位置を返す
		return self._doc.selection.range.get()[0]

	def set_caret_pos(self, pos):
		"""
		Set new caret position
		@type pos: int
		"""
		print "set_caret_pos"
		# キャレットを移動する
		self.create_selection(pos, pos)

	def get_current_line(self):
		"""
		Returns content of current line
		@return: str
		"""
		print "get_current_line"
		
		# 現在の行を選択する。
		self._doc.selection.line_range.set(to=self._doc.selection.line_range.get())
		
		# 現在行の選択範囲からテキストを得る
		res = self._doc.selection.contents.get()
		
		# 選択範囲を戻す
		self._doc.selection.range.set(to=self._original_selection_range)
		
		return res

	def replace_content(self, value, start=None, end=None, dummy=True):
		"""
		Replace editor's content or it's part (from <code>start</code> to
		<code>end</code> index). If <code>value</code> contains
		<code>caret_placeholder</code>, the editor will put caret into
		this position. If you skip <code>start</code> and <code>end</code>
		arguments, the whole target's content will be replaced with
		<code>value</code>.

		If you pass <code>start</code> argument only,
		the <code>value</code> will be placed at <code>start</code> string
		index of current content.

		If you pass <code>start</code> and <code>end</code> arguments,
		the corresponding substring of current target's content will be
		replaced with <code>value</code>
		@param value: Content you want to paste
		@type value: str
		@param start: Start index of editor's content
		@type start: int
		@param end: End index of editor's content
		@type end: int
		@param dummy: 最後の引数を渡す呼び出しをしているところがあるので追加。
		@type dummy: bool
		"""
		print "replace_content"
		if start == None :
			# 全て置き換え
			start = 0
			end   = get_content().len()
		else :
			if end == None :
				# start 位置から挿入
				end = start
		
		# value をエスケープする
		# value = re.sub("\"", "\\\"", value)
		
		#print "   ", start, end, value
		
		self.create_selection(start, end)
		
		# 値の挿入
		self._doc.selection.contents.set(to=value)
		
		# 最初のプレイスホルダを選択・削除
		self.create_selection(start, start)
		self._doc.replace(for_=zencoding.utils.caret_placeholder, to="")
		#self._doc.find(for_=zencoding.utils.caret_placeholder)
		
	def get_content(self):
		"""
		Returns editor's content
		@return: str
		"""
		print "get_content"
		# CotEditor の最前面の書類文字列を得る
		if self._content == None or self._content == '' :
			ret = self._doc.contents.get()
			#ret = unicode(ret,"utf-8")
			self._content = ret
		else :
			# 既に取得済み
			ret = self._content
		
		return ret
	
	def get_syntax(self):
		"""
		Returns current editor's syntax mode
		@return: str
		"""
		print "get_syntax"
		# CotEditor の最前面の書類の拡張子を得る
		thePath = self._doc.name.get()
		root, ext = os.path.splitext(thePath)
		x = re.sub("\.", "", ext)
		print "  %s" % x
		return x
	
	def set_profile_name(self, profilename):
		self._forcedProfile = profilename
		return
		
	def get_profile_name(self):
		"""
		Returns current output profile name (@see zen_coding#setup_profile)
		@return {String}
		"""
		print "get_profile_name"
		
		# 設定されていたらそのプロファイル名を返す
		if (self._forcedProfile != ''):
			print "  %s" % self._forcedProfile
			return self._forcedProfile
		
		# CotEditor の最前面の書類の拡張子を得る
		thePath = self._doc.name.get()
		root, ext = os.path.splitext(thePath)
		x = re.sub("\.", "", ext)
		if (x == 'xhtml' or x == 'html' or x == 'xml'):
			profilename = 'xhtml'
		elif (x == 'shtml'):
			profilename = 'html'
		else:
			profilename = 'plain'
		
		print "  %s" % profilename
		self._forcedProfile = profilename
		
		return profilename
	
	def prompt(self, title):
		"""
		Ask user to enter something
		@param title: Dialog title
		@type title: str
		@return: Entered data
		@since: 0.65
		"""
		# プロンプトを表示する
		print "prompt"
		result = commands.getoutput("""osascript -e '
			tell application "System Events" 
				activate
				set ret to display dialog "%s" default answer ""
				
				set buttonReturned to button returned of ret
				if buttonReturned = "OK" then
					return text returned of ret
				else
					return ""
				end if
			end tell
			'""" % (title))
		print result
		return result
	
	def get_selection(self):
		"""
		Returns current selection
		@return: str
		@since: 0.65
		"""
		print "get_selection"
		# 選択範囲のテキストを得る
		theSelection = self._doc.selection.contents.get()
		#theSelection = unicode(theSelection, "utf-8")
		
		return theSelection
		
		#以下、一時ファイルに保存する場合
		
		# 選択範囲のテキストを一時ファイルに書き込む
		thePath = "/tmp/zen_for_cot_selection." + self.get_profile_name()
		f = open(thePath, 'w')
		f.write(theSelection.encode('utf-8'))
		f.close
		
		return thePath
	
	def get_file_path(self):
		"""
		Returns current editor's file path
		@return: str
		@since: 0.65 
		"""
		print "get_file_path"
		# 最前面のドキュメントのパスを得る
		return self._doc.path.get()
	