# -*- coding: utf-8 -*-

'''
High-level editor interface that communicates with underlying editor (like
Espresso, Coda, etc.) or browser.
Basically, you should call <code>set_context(obj)</code> method to
set up undelying editor context before using any other method.

This interface is used by <i>zen_actions.py</i> for performing different
actions like <b>Expand abbreviation</b>

@example
from zencoding.interface.editor import ZenEditor
editor = ZenEditor(context)
//now you are ready to use editor object
editor.get_selection_range();

@author Sergey Chikuyonok (serge.che@gmail.com)
@link http://chikuyonok.ru

#CotEditor 専用に変更
#修正:Tachi.K
#修正日:2011/03/17

'''
import sys
import commands
import os.path
import re

class ZenEditor():
	def __init__(self, context=None):
		self._content = u''
		self._original_caret_range_start = None
		self._original_caret_range_end   = None
		self._original_selection_length  = None
		
		# テストコード
		fDoc = commands.getoutput("""osascript -e 'tell app "CotEditor" to get front document'""")
		print fDoc
		
		# テストコード
		
		# 現在の選択範囲を得る
		res = commands.getoutput("""osascript -e 'tell app "CotEditor" to get range of selection of front document'""")
		pos = res.split(",")
		self._original_caret_range_start = int(pos[0])
		self._original_caret_range_end   = self._original_caret_range_start + int(pos[1])
		self._original_selection_length  = self._original_caret_range_end - self._original_caret_range_start
		
		# CotEditor で編集中の全テキストを得る
		self.set_context(unicode(sys.stdin.read(), "utf-8"))

	def set_context(self, context=None):
		"""
		Setup underlying editor context. You should call this method
		<code>before</code> using any Zen Coding action.
		@param context: context object
		"""
		print "set_context"
		self._content = context

	def get_selection_range(self):
		"""
		Returns character indexes of selected text
		@return: list of start and end indexes
		@example
		start, end = zen_editor.get_selection_range();
		print('%s, %s' % (start, end))
		"""
		print "get_selection_range"
		# 選択範囲を得る
		return self._original_caret_range_start, self._original_caret_range_end


	def create_selection(self, start, end=None):
		"""
		Creates selection from <code>start</code> to <code>end</code> character
		indexes. If <code>end</code> is ommited, this method should place caret
		and <code>start</code> index
		@type start: int
		@type end: int
		@example
		zen_editor.create_selection(10, 40)
		# move caret to 15th character
		zen_editor.create_selection(15)
		"""
		print "create_selection"
		# 選択範囲を設定する
		if end == None :
			length = 0
		else :
			length = end-start
		
		commands.getoutput("""osascript -e 'tell app "CotEditor" to set range of selection of front document to {%d, %d}'""" % (start, length))

	def get_current_line_range(self):
		"""
		Returns current line's start and end indexes
		@return: list of start and end indexes
		@example
		start, end = zen_editor.get_current_line_range();
		print('%s, %s' % (start, end))
		"""
		print "get_current_line_range"
		
		# 現在の行を選択する。
		commands.getoutput("""osascript -e 'tell application "CotEditor" to set line range of selection of front document to get line range of selection'""")
		
		# 現在行の選択範囲を得る
		res = commands.getoutput("""osascript -e 'tell app "CotEditor" to get range of selection of front document'""")
		pos = res.split(",")
		start = int(pos[0])
		end   = start + int(pos[1])
		
		# 選択範囲を戻す
		commands.getoutput("""osascript -e 'tell app "CotEditor" to set range of selection of front document to {%d, %d}'""" % (self._original_caret_range_start, self._original_selection_length))
		
		return start, end

	def get_caret_pos(self):
		""" Returns current caret position """
		print "get_caret_pos"
		# 現在のキャレット位置を返す
		return self._original_caret_range_start

	def set_caret_pos(self, pos):
		"""
		Set new caret position
		@type pos: int
		"""
		print "set_caret_pos"
		# キャレットを移動する
		self.create_selection(pos, pos)

	def get_current_line(self):
		"""
		Returns content of current line
		@return: str
		"""
		print "get_current_line"
		
		# 現在の行を選択する。
		commands.getoutput("""osascript -e 'tell application "CotEditor" to set line range of selection of front document to get line range of selection'""")
		
		# 現在行の選択範囲からテキストを得る
		res = commands.getoutput("""osascript -e 'tell app "CotEditor" to get contents of selection of front document'""")
		
		# 選択範囲を戻す
		commands.getoutput("""osascript -e 'tell app "CotEditor" to set range of selection of front document to {%d, %d}'""" % (self._original_caret_range_start, self._original_selection_length))
		
		return res

	def replace_content(self, value, start=None, end=None):
		"""
		Replace editor's content or it's part (from <code>start</code> to
		<code>end</code> index). If <code>value</code> contains
		<code>caret_placeholder</code>, the editor will put caret into
		this position. If you skip <code>start</code> and <code>end</code>
		arguments, the whole target's content will be replaced with
		<code>value</code>.

		If you pass <code>start</code> argument only,
		the <code>value</code> will be placed at <code>start</code> string
		index of current content.

		If you pass <code>start</code> and <code>end</code> arguments,
		the corresponding substring of current target's content will be
		replaced with <code>value</code>
		@param value: Content you want to paste
		@type value: str
		@param start: Start index of editor's content
		@type start: int
		@param end: End index of editor's content
		@type end: int
		"""
		print "replace_content"
		if start == None :
			# 全て置き換え
			start = 0
			end   = get_content().len()
		else :
			if end == None :
				# start 位置から挿入
				end = start
		
		# value をエスケープする
		value = re.sub("\"", "\\\"", value)
		
		print "   ", start, end, value
		
		self.create_selection(start, end)
		
		# 値の挿入
		commands.getoutput("""osascript -e 'tell app "CotEditor" to set contents of selection of front document to "%s"'""" % (value))
		
		# 最初のプレイスホルダを選択
		self.create_selection(start, start)
		commands.getoutput("""osascript -e 'tell app "CotEditor" to tell front document to find for "<##>"'""")
	
	def get_content(self):
		"""
		Returns editor's content
		@return: str
		"""
		print "get_content"
		# CotEditor の最前面の書類文字列を得る
		if self._content == None or self._content == '' :
			ret = commands.getoutput("""osascript -e 'tell app "CotEditor" to get contents of front document'""")
			ret = unicode(ret,"utf-8")
		else :
			# 既に取得済み
			ret = self._content
		
		return ret
	
	def get_syntax(self):
		"""
		Returns current editor's syntax mode
		@return: str
		"""
		print "get_syntax"
		# CotEditor の最前面の書類の拡張子を得る
		thePath = commands.getoutput("""osascript -e 'tell app "CotEditor" to get path of front document'""")
		root, ext = os.path.splitext(thePath)
		x = re.sub("\.", "", ext)
		print "  %s" % ext
		return ext
	
	def get_profile_name(self):
		"""
		Returns current output profile name (@see zen_coding#setup_profile)
		@return {String}
		"""
		print "get_profile_name"
		
		ret = ''
		return ret
	
	def prompt(self, title):
		"""
		Ask user to enter something
		@param title: Dialog title
		@type title: str
		@return: Entered data
		@since: 0.65
		"""
		# プロンプトを表示する
		print "prompt"
		return commands.getoutput("""osascript -e 'tell application "System Events" to display dialog "%a" default answer ""'""" % (title))
	
	def get_selection(self):
		"""
		Returns current selection
		@return: str
		@since: 0.65
		"""
		print "get_selection"
		# 選択範囲のテキストを得る
		ret = commands.getoutput("""osascript -e 'tell app "CotEditor" to get contents of selection of front document'""")
		ret = unicode(ret, "utf-8")
		return ret
	
	def get_file_path(self):
		"""
		Returns current editor's file path
		@return: str
		@since: 0.65 
		"""
		print "get_file_path"
		# 最前面のドキュメントのパスを得る
		return commands.getoutput("""osascript -e 'tell app "CotEditor" to get path of front document'""")
	